<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use DateTime;
use JsonSerializable;

/**
 * File entity
 */
final class File implements JsonSerializable {

	/**
	 * @var string File name
	 */
	private $name;

	/**
	 * @var int File size
	 */
	private $size;

	/**
	 * @var DateTime File creation date
	 */
	private $date;

	/**
	 * @var string SHA256 hash of the file
	 */
	private $sha256;

	/**
	 * Constructor
	 * @param string $name File name
	 * @param int $size File size
	 * @param int $date File creation date
	 * @param string $sha256 SHA256 hash of the file
	 */
	public function __construct(string $name, int $size, int $date, string $sha256) {
		$this->name = ltrim($name, './');
		$this->size = $size;
		$this->date = (new DateTime())->setTimestamp($date);
		$this->sha256 = $sha256;
	}

	/**
	 * Creates a new file entity from an array
	 * @param array<string,mixed> $data JSON data in an array
	 * @return File File entity
	 */
	public static function fromArray(array $data): self {
		return new static($data['name'], $data['size'], $data['date'], $data['sha256']);
	}

	/**
	 * Returns the file name
	 * @return string File name
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * Returns the file size
	 * @return int File size
	 */
	public function getSize(): int {
		return $this->size;
	}

	/**
	 * Returns the file creation date
	 * @return DateTime File creation date
	 */
	public function getDate(): DateTime {
		return $this->date;
	}

	/**
	 * Returns the SHA256 hash of the file
	 * @return string SHA256 hash of the file
	 */
	public function getSha256(): string {
		return $this->sha256;
	}

	/**
	 * Serializes the File entity into JSON
	 * @return array<string, int|string> JSON serialized File entity
	 */
	public function jsonSerialize(): array {
		return [
			'name' => $this->name,
			'size' => $this->size,
			'creationDate' => $this->date->format('c'),
			'sha256' => $this->sha256,
		];
	}

}
