<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use JsonSerializable;

/**
 * Manufacturer entity
 */
final class Manufacturer implements JsonSerializable {

	/**
	 * @var int Manufacturer ID
	 */
	private $id;

	/**
	 * @var int Company ID
	 */
	private $companyId;

	/**
	 * @var string Manufacturer name
	 */
	private $name;

	/**
	 * Constructor
	 * @param int $id Manufacturer name
	 * @param int $companyId Company ID
	 * @param string $name Manufacturer name
	 */
	public function __construct(int $id, int $companyId, string $name) {
		$this->id = $id;
		$this->name = $name;
		$this->companyId = $companyId;
	}

	/**
	 * Creates a new manufacturer entity from an array
	 * @param array<string,mixed> $data JSON data in an array
	 * @return Manufacturer Created manufacturer entity from an array
	 */
	public static function fromArray(array $data): self {
		return new static($data['manufacturerID'], $data['companyID'], $data['name']);
	}

	/**
	 * Returns company ID
	 * @return int Company ID
	 */
	public function getCompanyId(): int {
		return $this->companyId;
	}

	/**
	 * Returns manufacturer ID
	 * @return int Manufacturer ID
	 */
	public function getId(): int {
		return $this->id;
	}

	/**
	 * Returns manufacturer name
	 * @return string Manufacturer name
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * Serializes the Manufacturer entity into JSON
	 * @return array<string, int|string> JSON serialized Manufacturer entity
	 */
	public function jsonSerialize(): array {
		return [
			'id' => $this->id,
			'name' => $this->name,
			'companyId' => $this->companyId,
		];
	}

}
