<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

/**
 * Package entity
 */
final class Package {

	/**
	 * @var int Package ID
	 */
	private $id;

	/**
	 * @var int Package HWPID
	 */
	private $hwpid;

	/**
	 * @var int Package HWPID version
	 */
	private $hwpidVer;

	/**
	 * @var string Custom handler URL
	 */
	private $handlerUrl;

	/**
	 * @var string Custom handler hash
	 */
	private $handlerHash;

	/**
	 * @var string IQRF OS build
	 */
	private $os;

	/**
	 * @var string DPA version
	 */
	private $dpa;

	/**
	 * @var string Package notes
	 */
	private $notes;

	/**
	 * Constructor
	 * @param int $id Package ID
	 * @param int $hwpid HWPID
	 * @param int $hwpidVer HWPID version
	 * @param string $handlerUrl Custom handler URL
	 * @param string $handlerHash Custom handler hash
	 * @param string $os IQRF OS build
	 * @param string $dpa DPA version
	 * @param string $notes Package notes
	 */
	public function __construct(int $id, int $hwpid, int $hwpidVer, string $handlerUrl, string $handlerHash, string $os, string $dpa, string $notes) {
		$this->id = $id;
		$this->hwpid = $hwpid;
		$this->hwpidVer = $hwpidVer;
		$this->handlerUrl = $handlerUrl;
		$this->handlerHash = $handlerHash;
		$this->os = $os;
		$this->dpa = $dpa;
		$this->notes = $notes;
	}

	/**
	 * Creates a new package entity from an array
	 * @param array<string,mixed> $data JSON data in an array
	 * @return Package Created package entity
	 */
	public static function fromArray(array $data): self {
		return new static($data['packageID'], $data['hwpid'], $data['hwpidVer'], $data['handlerUrl'], $data['handlerHash'], $data['os'], $data['dpa'], $data['notes']);
	}

	/**
	 * Returns the package ID
	 * @return int Package ID
	 */
	public function getId(): int {
		return $this->id;
	}

	/**
	 * Returns the package HWPID
	 * @return int package HWPID
	 */
	public function getHwpid(): int {
		return $this->hwpid;
	}

	/**
	 * Returns the package HWPID version
	 * @return int Package HWPID version
	 */
	public function getHwpidVer(): int {
		return $this->hwpidVer;
	}

	/**
	 * Returns the package custom handler URL
	 * @return string Package custom handler URL
	 */
	public function getHandlerUrl(): string {
		return $this->handlerUrl;
	}

	/**
	 * Returns the package custom handler hash
	 * @return string Package custom handler hash
	 */
	public function getHandlerHash(): string {
		return $this->handlerHash;
	}

	/**
	 * Returns IQRF OS build
	 * @return string IQRF OS build
	 */
	public function getOs(): string {
		return $this->os;
	}

	/**
	 * Returns DPA version
	 * @return string DPA version
	 */
	public function getDpa(): string {
		return $this->dpa;
	}

	/**
	 * Returns the package notes
	 * @return string Package notes
	 */
	public function getNotes(): string {
		return $this->notes;
	}

}
