<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use Iqrf\Repository\Enums\RfModes;

/**
 * Product entity
 */
final class Product {

	/**
	 * @var int Product HWPID
	 */
	private $hwpid;

	/**
	 * @var string Product name
	 */
	private $name;

	/**
	 * @var int Manufacturer ID
	 */
	private $manufacturerId;

	/**
	 * @var string Company name
	 */
	private $companyName;

	/**
	 * @var string Product homepage
	 */
	private $homepage;

	/**
	 * @var string Product picture
	 */
	private $picture;

	/**
	 * @var RfModes Product RF mode
	 */
	private $rfMode;

	/**
	 * Constructor
	 * @param int $hwpid Product HWPID
	 * @param string $name Product name
	 * @param int $manufacturerID Manufacturer ID
	 * @param string $companyName Company name
	 * @param string $homePage Product homepage
	 * @param string $picture Product picture
	 * @param int $rfMode Product RF mode
	 */
	public function __construct(int $hwpid, string $name, int $manufacturerID, string $companyName, string $homePage, string $picture, int $rfMode) {
		$this->hwpid = $hwpid;
		$this->name = $name;
		$this->manufacturerId = $manufacturerID;
		$this->companyName = $companyName;
		$this->homepage = $homePage;
		$this->picture = $picture;
		$this->rfMode = RfModes::fromScalar($rfMode);
	}

	/**
	 * Creates a new product entity from an array
	 * @param array<string,mixed> $data JSON data in an array
	 * @return Product Created product entity
	 */
	public static function fromArray(array $data): self {
		return new static($data['hwpid'], $data['name'], $data['manufacturerID'], $data['companyName'], $data['homePage'], $data['picture'], $data['rfMode']);
	}

	/**
	 * Returns the product HWPID
	 * @return int Product HWPID
	 */
	public function getHwpid(): int {
		return $this->hwpid;
	}

	/**
	 * Returns the product name
	 * @return string Product name
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * Returns the manufacturer ID
	 * @return int Manufacturer ID
	 */
	public function getManufacturerId(): int {
		return $this->manufacturerId;
	}

	/**
	 * Returns the company name
	 * @return string Company name
	 */
	public function getCompanyName(): string {
		return $this->companyName;
	}

	/**
	 * Returns the product homepage
	 * @return string Product homepage
	 */
	public function getHomepage(): string {
		return $this->homepage;
	}

	/**
	 * Returns the product picture
	 * @return string Product picture
	 */
	public function getPicture(): string {
		return $this->picture;
	}

	/**
	 * Returns the product RF mode
	 * @return RfModes Product RF mode
	 */
	public function getRfMode(): RfModes {
		return $this->rfMode;
	}

}
