<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use JsonSerializable;

/**
 * Detailed IQRF Standard entity
 */
final class StandardDetail implements JsonSerializable {

	/**
	 * @var int Standard ID
	 */
	private $id;

	/**
	 * @var string Standard name
	 */
	private $name;

	/**
	 * @var float Standard version
	 */
	private $version;

	/**
	 * @var int Flags for this version
	 */
	private $versionFlags;

	/**
	 * @var string Driver code
	 */
	private $driver;

	/**
	 * @var string Notes
	 */
	private $notes;

	/**
	 * Constructor
	 * @param int $id Standard ID
	 * @param string $name Standard name
	 * @param float $version Standard version
	 * @param int $versionFlags Flags for this version
	 * @param string $driver Driver code
	 * @param string $notes Notes
	 */
	public function __construct(int $id, string $name, float $version, int $versionFlags, string $driver, string $notes) {
		$this->id = $id;
		$this->name = $name;
		$this->version = $version;
		$this->versionFlags = $versionFlags;
		$this->driver = $driver;
		$this->notes = $notes;
	}

	/**
	 * Creates an new detailed IQRF Standard entity from an array
	 * @param array<string, mixed> $data JSON data an an array
	 * @return StandardDetail Created detailed IQRF Standard entity
	 */
	public static function fromArray(array $data): self {
		return new static($data['standardID'], $data['name'], $data['version'], $data['versionFlags'], $data['driver'], $data['notes']);
	}

	/**
	 * Returns the standard ID
	 * @return int Standard ID
	 */
	public function getId(): int {
		return $this->id;
	}

	/**
	 * Returns the standard name
	 * @return string Standard name
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * Returns the standard version
	 * @return float Standard versions
	 */
	public function getVersion(): float {
		return $this->version;
	}

	/**
	 * Returns the flags for this version
	 * @return int Flags for this version
	 */
	public function getVersionFlags(): int {
		return $this->versionFlags;
	}

	/**
	 * Returns the driver code
	 * @return string Driver code
	 */
	public function getDriver(): string {
		return $this->driver;
	}

	/**
	 * Returns the notes
	 * @return string Notes
	 */
	public function getNotes(): string {
		return $this->notes;
	}

	/**
	 * Serializes the detailed IQRF Standard entity into JSON
	 * @return array<string, float|int|string> JSON serialized detailed IQRF Standard entity
	 */
	public function jsonSerialize(): array {
		return [
			'id' => $this->id,
			'name' => $this->name,
			'version' => $this->version,
			'versionFlags' => $this->versionFlags,
			'driver' => $this->driver,
			'notes' => $this->notes,
		];
	}

}
