<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use JsonSerializable;

/**
 * IQRF Standard versions entity
 */
final class StandardVersions implements JsonSerializable {

	/**
	 * @var int Standard ID
	 */
	private $id;

	/**
	 * @var string Standard name
	 */
	private $name;

	/**
	 * @var array<float> Array of available versions
	 */
	private $versions;

	/**
	 * Constructor
	 * @param int $id Standard ID
	 * @param string $name Standard name
	 * @param array<float> $versions Array of available versions
	 */
	public function __construct(int $id, string $name, array $versions) {
		$this->id = $id;
		$this->name = $name;
		$this->versions = $versions;
	}

	/**
	 * Creates an new IQRF Standard with all avaiable versions entity
	 * @param array<string,mixed> $data JSON data in an array
	 * @return StandardVersions Created IQRF Standard with all available versions entity
	 */
	public static function fromArray(array $data): self {
		return new static($data['standardID'], $data['name'], $data['versions']);
	}

	/**
	 * Returns the standard ID
	 * @return int Standard ID
	 */
	public function getId(): int {
		return $this->id;
	}

	/**
	 * Returns the standard name
	 * @return string Standard name
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * Returns the array of available versions
	 * @return array<float> Array of available versions
	 */
	public function getVersions(): array {
		return $this->versions;
	}

	/**
	 * Serializes Standard versions entity into JSON
	 * @return array<string, array<float>|int|string> JSON serialized Standard versions entity
	 */
	public function jsonSerialize(): array {
		return [
			'id' => $this->id,
			'name' => $this->name,
			'versions' => $this->versions,
		];
	}

}
