<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Utils;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Iqrf\Repository\Configuration;
use Iqrf\Repository\Exceptions\CorruptedData;
use Iqrf\Repository\Exceptions\DataNotFound;
use Iqrf\Repository\Exceptions\ServiceUnavailable;
use Nette\Utils\Json;
use Nette\Utils\JsonException;

/**
 * HTTP(S) client for API
 */
class ApiClient {

	/**
	 * @var Configuration Configuration
	 */
	private $configuration;

	/**
	 * @var Client HTTP(S) client
	 */
	private $httpClient;

	/**
	 * Constructor
	 * @param Configuration $config Configuration
	 * @param Client $httpClient HTTP(S) client
	 */
	public function __construct(Configuration $config, Client $httpClient) {
		$this->configuration = $config;
		$this->httpClient = $httpClient;
	}

	/**
	 * Sends JSON API request
	 * @param string $path API path
	 * @param array<mixed>|null $query Optional API query
	 * @return array<mixed> JSON API response
	 * @throws ServiceUnavailable
	 */
	public function sendRequest(string $path, ?array $query = null): array {
		$url = $this->configuration->getApiEndpoint() . '/' . $path;
		if ($query === null) {
			$options = [];
		} else {
			$options = ['query' => $query];
		}
		try {
			$content = $this->httpClient->get($url, $options)->getBody()->getContents();
			return Json::decode($content, Json::FORCE_ARRAY);
		} catch (RequestException $e) {
			if ($e->getCode() === 404) {
				throw new DataNotFound();
			}
			throw new ServiceUnavailable();
		} catch (JsonException $e) {
			throw new CorruptedData();
		}
	}

}
